"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packager = exports.sort = exports.deserialize = void 0;
const debug_1 = __importDefault(require("debug"));
const global_ids_1 = require("./global-ids");
const gql_1 = require("./gql");
const query_1 = require("./gql/query");
const minilinks_1 = require("./minilinks");
const promise_1 = require("./promise");
const reserve_1 = require("./reserve");
const debug = (0, debug_1.default)('deeplinks:packager');
function deserialize(pckg, errors = []) {
    const data = pckg.data.map(l => (Object.assign(Object.assign({}, l), { value: l.value ? Object.assign({}, l.value) : undefined })));
    const containsHash = {};
    let counter = 0;
    for (let l = 0; l < data.length; l++) {
        const item = data[l];
        const local = item.id;
        if (containsHash[local])
            item.id = containsHash[local];
        else
            containsHash[local] = item.id = ++counter;
        item._ = local;
    }
    for (let l = 0; l < data.length; l++) {
        const item = data[l];
        if (item.type)
            item.type = containsHash[item.type];
        if (item.from)
            item.from = containsHash[item.from] || 0;
        if (item.to)
            item.to = containsHash[item.to] || 0;
    }
    const containsArray = Object.keys(containsHash);
    for (let c = 0; c < containsArray.length; c++) {
        const contain = containsArray[c];
        data.push({
            id: ++counter,
            type: containsHash === null || containsHash === void 0 ? void 0 : containsHash['Contain'],
            from: containsHash === null || containsHash === void 0 ? void 0 : containsHash['package'],
            to: containsHash[contain],
            value: { value: contain },
        });
    }
    return { data, errors, counter };
}
exports.deserialize = deserialize;
function sort(pckg, data, errors = []) {
    let sorted = [];
    if (pckg.strict) {
        sorted = data;
    }
    else {
        for (let i = 0; i < data.length; i++) {
            const item = data[i];
            if (item === null || item === void 0 ? void 0 : item.type) {
                const first = sorted.findIndex((l, index) => l.from == item.id || l.to == item.id || l.type == item.id);
                const max = sorted.reduce((p, l, index) => l.id == item.type || l.id == item.from || l.id === item.to ? index : p, 0);
                sorted.splice((!!~first && first < max ? first : max) + 1, 0, item);
            }
            else {
                const max = sorted.reduce((p, l, index) => l.id == item.id ? index : p, 0);
                sorted.splice((max) + 1, 0, item);
            }
        }
    }
    return { sorted };
}
exports.sort = sort;
class Packager {
    constructor(client) {
        this.client = client;
        global.packager = this;
    }
    fetchTableNamesValuedToTypeId(types, errors) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = {};
            try {
                const q = yield this.client.query((0, query_1.generateQuery)({
                    queries: [
                        (0, query_1.generateQueryData)({ tableName: 'links', returning: `id values: out(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_TABLE_VALUE} } }) { id: to_id }`, variables: { where: {
                                    type_id: { _eq: global_ids_1.GLOBAL_ID_TABLE },
                                    out: { type_id: { _eq: global_ids_1.GLOBAL_ID_TABLE_VALUE }, to_id: { _in: types } },
                                } } }),
                    ],
                    name: 'LOAD_TYPES_TABLES_HASH',
                }));
                const tables = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0;
                for (let t = 0; t < tables.length; t++) {
                    const table = tables[t];
                    for (let i = 0; i < table.values.length; i++) {
                        const type = table.values[i].id;
                        if (types.includes(type))
                            result[type] = table.id;
                    }
                }
            }
            catch (error) {
                debug('fetchTableNamesValuedToTypeId error', error);
                errors.push(error);
            }
            return result;
        });
    }
    insertItem(id, items, item, errors, mutated) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('insertItem', { id }, item);
            try {
                if (item.type) {
                    const linkInsert = yield this.client.mutate((0, gql_1.generateSerial)({
                        actions: [(0, gql_1.insertMutation)('links', { objects: { id, type_id: item.type, from_id: item.from || 0, to_id: item.to || 0 } })],
                        name: 'IMPORT_PACKAGE_LINK',
                    }));
                    if (linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors) {
                        errors.push(linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors);
                    }
                    mutated[item.id] = true;
                    item.id = id;
                    items.filter(i => i.id === item.id).forEach(l => l.id = id);
                    items.filter(i => i.from === item.id).forEach(l => l.from = id);
                    items.filter(i => i.to === item.id).forEach(l => l.to = id);
                    items.filter(i => i.type === item.id).forEach(l => l.type = id);
                }
                debug('insertItem promise', id);
                yield (0, promise_1.awaitPromise)({ id, client: this.client });
                debug('insertItem promise awaited', id);
                if (item.value) {
                    debug('insertItem value', id, item);
                    let valueLink;
                    if (!item.type) {
                        valueLink = items.find(i => i.id === item.id && !!i.type);
                        debug('insertItem .value', item, valueLink);
                    }
                    else {
                        valueLink = item;
                    }
                    if (!valueLink) {
                        errors.push(`Link ${item.id} for value not founded.`);
                    }
                    else {
                        const tables = yield this.fetchTableNamesValuedToTypeId([+valueLink.type], errors);
                        debug('insertItem tables', tables);
                        if (!tables[valueLink.type])
                            errors.push(`Table for type ${valueLink.type} for link ${valueLink.id} not founded for insert ${JSON.stringify(item)}.`);
                        const valueInsert = yield this.client.mutate((0, gql_1.generateSerial)({
                            actions: [(0, gql_1.insertMutation)(`table${tables[valueLink.type]}`, { objects: Object.assign({ link_id: valueLink.id }, item.value) })],
                            name: 'IMPORT_PACKAGE_VALUE',
                        }));
                        debug('insertItem valueInsert', valueInsert);
                        if (valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors)
                            errors.push(valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors);
                    }
                }
            }
            catch (error) {
                debug('insertItem error', error);
                errors.push(error);
            }
            return;
        });
    }
    insertItems(pckg, data, counter, errors = [], mutated = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const ids = yield (0, reserve_1.reserve)({
                    client: this.client,
                    count: counter,
                });
                for (let i = 0; i < data.length; i++) {
                    const item = data[i];
                    const id = ids[+(item.id) - 1];
                    if (!id)
                        console.log(`Cant find id for`, item, data.filter(i => i.id === item.id));
                    yield this.insertItem(id, data, item, errors, mutated);
                    if (errors.length)
                        return;
                }
            }
            catch (error) {
                debug('insertItems error', error);
                errors.push(error);
            }
        });
    }
    import(pckg) {
        return __awaiter(this, void 0, void 0, function* () {
            const errors = [];
            try {
                const { data, counter } = deserialize(pckg, errors);
                if (errors.length)
                    return { errors };
                const { sorted } = sort(pckg, data, errors);
                if (errors.length)
                    return { errors };
                yield this.insertItems(pckg, sorted, counter, errors);
                if (errors.length)
                    return { errors };
            }
            catch (error) {
                debug('insertItems error', error);
                errors.push(error);
            }
            return { errors };
        });
    }
    selectLinks(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.query((0, query_1.generateQuery)({
                queries: [
                    (0, query_1.generateQueryData)({ tableName: 'links', returning: `
          id type_id from_id to_id value
          type {
            id value
            contains: in(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_CONTAIN} }, from: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_PACKAGE} } } }) {
              id
              package: from {
                id value
              }
            }
          }
          from {
            id
            contains: in(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_CONTAIN} }, from: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_PACKAGE} } } }) {
              id
              package: from {
                id value
              }
            }
          }
          to {
            id
            contains: in(where: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_CONTAIN} }, from: { type_id: { _eq: ${global_ids_1.GLOBAL_ID_PACKAGE} } } }) {
              id
              package: from {
                id value
              }
            }
          }
        `, variables: { where: {
                                _or: [
                                    { id: { _eq: options.packageLinkId } },
                                    { type_id: { _eq: global_ids_1.GLOBAL_ID_CONTAIN }, from: { id: { _eq: options.packageLinkId } } },
                                    { in: { type_id: { _eq: global_ids_1.GLOBAL_ID_CONTAIN }, from: { id: { _eq: options.packageLinkId } } } },
                                ]
                            } } }),
                ],
                name: 'LOAD_PACKAGE_LINKS',
            }));
            return (0, minilinks_1.minilinks)((_b = (_a = (result)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0);
        });
    }
    serialize(globalLinks, options, pckg) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let counter = 1;
            const containsByTo = {};
            const links = [];
            for (let l = 0; l < globalLinks.links.length; l++) {
                const link = globalLinks.links[l];
                if (link.type_id === global_ids_1.GLOBAL_ID_CONTAIN) {
                    containsByTo[+link.to_id] = link;
                }
                else
                    links.push(link);
            }
            for (let l = 0; l < links.length; l++) {
                const link = links[l];
                link._id = link.id;
                link.id = ((_b = (_a = containsByTo[+link.id]) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) ? containsByTo[+link.id].value.value : counter++;
                for (let li = 0; li < link.out.length; li++) {
                    const _link = link.out[li];
                    _link.from_id = link.id;
                }
                for (let li = 0; li < link.in.length; li++) {
                    const _link = link.in[li];
                    _link.to_id = link.id;
                }
                for (let li = 0; li < link.typed.length; li++) {
                    const _link = link.typed[li];
                    _link.type_id = link.id;
                }
                let value;
                if (link === null || link === void 0 ? void 0 : link.value) {
                    const _c = link.value, { id: __id, link_id: __link_id } = _c, _value = __rest(_c, ["id", "link_id"]);
                    value = _value;
                }
                const newLink = {
                    id: link.id,
                    type: link.type_id,
                };
                if (link.from)
                    newLink.from = link.from.id;
                if (link.to)
                    newLink.to = link.to.id;
                if (value)
                    newLink.value = value;
                pckg.data.push(newLink);
            }
        });
    }
    export(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const globalLinks = yield this.selectLinks(options);
            const packageLink = (_a = globalLinks.links) === null || _a === void 0 ? void 0 : _a.find(l => { var _a, _b; return ((_b = (_a = l === null || l === void 0 ? void 0 : l.type) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) === global_ids_1.GLOBAL_NAME_PACKAGE; });
            const packageName = (_b = packageLink === null || packageLink === void 0 ? void 0 : packageLink.value) === null || _b === void 0 ? void 0 : _b.value;
            const pckg = {
                package: packageName,
                data: [],
                strict: false,
                errors: [],
            };
            yield this.serialize(globalLinks, options, pckg);
            return pckg;
        });
    }
}
exports.Packager = Packager;
//# sourceMappingURL=packager.js.map